@@ -218,7 +218,7 @@ class OptResultDef(VarResultDef, OptionalDef):
 
 
 @dataclass
-class RegionDef:
+class RegionDef(Region):
     """
     An IRDL region definition.
     If the block_args is specified, then the region expect to have the entry block with these arguments.
@@ -240,13 +240,22 @@ class AttributeDef:
     constr: AttrConstraint
     """The attribute constraint."""
 
-    data: Any
+    data: typing.Any
 
     def __init__(self, typ: Union[Attribute, typing.Type[Attribute],
                                   AttrConstraint]):
         self.constr = attr_constr_coercion(typ)
 
 
+@dataclass(init=False)
+class OptAttributeDef(AttributeDef):
+    """An IRDL attribute definition for an optional attribute."""
+
+    def __init__(self, typ: Union[Attribute, typing.Type[Attribute],
+                                  AttrConstraint]):
+        super().__init__(typ)
+
+
 def get_variadic_sizes(op: Operation, is_operand: bool) -> List[int]:
     """Get variadic sizes of operands or results."""
 
@@ -412,6 +421,8 @@ def irdl_op_verify(op: Operation, operands: List[Tuple[str, OperandDef]],
 
     for attr_name, attr_def in attributes:
         if attr_name not in op.attributes:
+            if isinstance(attr_def, OptAttributeDef):
+                continue
             raise Exception(f"attribute {attr_name} expected")
         attr_def.constr.verify(op.attributes[attr_name])
 
@@ -579,9 +590,15 @@ def irdl_op_definition(
         new_attrs[region_name] = property(
             (lambda idx: lambda self: self.regions[idx])(region_idx))
 
-    for attribute_name, _ in attr_defs:
-        new_attrs[attribute_name] = property(
-            (lambda name: lambda self: self.attributes[name])(attribute_name))
+    for attribute_name, attr_def in attr_defs:
+        if isinstance(attr_def, OptAttributeDef):
+            new_attrs[attribute_name] = property(
+                (lambda name: lambda self: self.attributes.get(name, None)
+                 )(attribute_name))
+        else:
+            new_attrs[attribute_name] = property(
+                (lambda name: lambda self: self.attributes[name]
+                 )(attribute_name))
 
     new_attrs["irdl_operand_defs"] = operand_defs
     new_attrs["irdl_result_defs"] = result_defs
@@ -614,7 +631,7 @@ def irdl_op_definition(
 
     new_attrs["build"] = classmethod(builder)
 
-    return type(cls.__name__, (cls, ), {**cls.__dict__, **new_attrs})
+    return type(cls.__name__, cls.__mro__, {**cls.__dict__, **new_attrs})
 
 
 @dataclass
