@@ -169,24 +169,23 @@ class Parser:
             block = self._blocks[block_name]
         else:
             block = Block()
-            if self.parse_optional_char("("):
-                tuple_list = self.parse_list(
-                    self.parse_optional_block_argument)
-                # TODO can we clean this up a bit?
-                # Register the BlockArguments as ssa values and add them to
-                # the block
-                for (idx, res) in enumerate(tuple_list):
-                    if res[0] in self._ssaValues:
-                        raise Exception("SSA value %s is already defined" %
-                                        res[0])
-                    arg = res[1]
-                    self._ssaValues[res[0]] = arg
-                    arg.index = idx
-                    arg.block = block
-                    block.args.append(arg)
-
-                self.parse_char(")")
             self._blocks[block_name] = block
+
+        if self.parse_optional_char("("):
+            tuple_list = self.parse_list(self.parse_optional_block_argument)
+            # TODO can we clean this up a bit?
+            # Register the BlockArguments as ssa values and add them to
+            # the block
+            for (idx, res) in enumerate(tuple_list):
+                if res[0] in self._ssaValues:
+                    raise Exception("SSA value %s is already defined" % res[0])
+                arg = res[1]
+                self._ssaValues[res[0]] = arg
+                arg.index = idx
+                arg.block = block
+                block.args.append(arg)
+
+            self.parse_char(")")
         self.parse_char(":")
         for op in self.parse_list(self.parse_optional_op, delimiter=""):
             block.add_op(op)
